<?php
/*
	Plugin Name: Bookable Events
	Plugin URI: http://provaqui.net84.net/wp/?p=176
	Description: This plugin allows you to create a bookable event using the custom fields. A post creator can make a post 'bookable' selecting the custom field 'bookable_event' (automatically created at the plugin activation) and setting a value for it, e.g. the 'yes' value.
	The registered users can add/remove their participation to an event (represented by a post) by links at the bottom of the post.
	Version: 1.0
	Author: Francesca 'sefran' Secondo 
	Author URI: http://provaqui.net84.net/wp
*/

/*
	Copyright 2009  Francesca Secondo  (email : sefran2 (at) gmail.com)

	This program is free software; you can redistribute it and/or modify
	it under the terms of the GNU General Public License as published by
	the Free Software Foundation; either version 2 of the License, or
	(at your option) any later version.
	
	This program is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
	GNU General Public License for more details.
	
	You should have received a copy of the GNU General Public License
	along with this program; if not, write to the Free Software
	Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
*/

/*
	Global vars for this plugin:
	- $bookable_key: name of the custom field used to make a post 'bookable'. This custom field have to be unique.
	- $participant_key: name of the custom field used to memorize the event participants' username. This custom field can be not unique.
	They are declared 'global' so the functions called by register_activation_hook() and register_deactivation_hook() have access to global variables.

	- $bookable: true if the post is 'bookable'.
	- $click_register: true if the user clicks on the registration link.
	- $click_unregister: true if the user clicks on the unregistration link.
	- $postid_clicked: post ID the user registers/unregisters for/from
*/

global $bookable_key;
global $participant_key;
$bookable_key = 'bookable_event';
$participant_key = 'participant'; 

$bookable;
$click_register = $_GET['click_register'];
$click_unregister = $_GET['click_unregister'];
$postid_clicked = $_GET['post'];

/*
	Internationalization
*/
load_plugin_textdomain( 'bookable_events', 'wp-content/plugins/bookable_events/lang' );


/* 
	Function to be run when the plugin is activated. 
	It adds the custom field 'bookable_event', in this way the post creator can select it (setting any value for it), making the post 'bookable'.
*/
function bookable_events_activate() {
	global $bookable_key;
	$bookable_value = '';
	$unique = true; // There can be only one custom field named 'bookable_event' e 'num_days'
	
	$allposts = get_posts('numberposts=0&post_type=post&post_status=');
	
	foreach( $allposts as $postinfo ) {
		add_post_meta( $postinfo->ID, $bookable_key, $bookable_value, $unique );
    }
}

/* 
	Function to be run when the plugin is deactivated. 
	It removes all the custom fields added by the plugin ('bookable_event','participant','num_days').
*/
function bookable_events_deactivate() {
	global $bookable_key;
	global $participant_key;

	$allposts = get_posts('numberposts=0&post_type=post&post_status=');
	
	foreach( $allposts as $postinfo ) {
		delete_post_meta( $postinfo->ID , $bookable_key );
		delete_post_meta( $postinfo->ID , $participant_key );
    }
}

/*
	Returns the xhtml registration link.
*/
function add_link_registration() {
	return __("If you want to register for this event, click on the following link after login.", 'bookable_events')."<br/><a href='$PHP_SELF?click_register=true&post=".get_the_ID()."'>".__("Register!", 'bookable_events')."</a>";
}

/*
	Returns the xhtml unregistration link.
*/
function add_link_unregistration() {
	return __("If you want to unregister from this event, click on the following link after login.", 'bookable_events')."<br/><a href='$PHP_SELF?click_unregister=true&post=".get_the_ID()."'>".__("Unregister!", 'bookable_events')."</a>";
}

/*
	Checks if a post is bookable.
*/
function is_bookable() {
	global $bookable_key;
	global $bookable;
	$meta = get_post_custom();
	$meta_book_event = $meta[$bookable_key][0];

	if ( $meta_book_event != '' ) return $bookable = true;
	else return $bookable = false;
}

/*
	Adds the registration/unregistration links at the bottom of the 'bookable' post.
*/
function add_links_to_post( $content = '' ) {
	if ( is_bookable() ) $content = "$content<br/><hr/>".add_link_registration()."<br/>".add_link_unregistration()."<br/>";
	return $content;
}

/*
	Adds the 'bookable event' label to the title of the 'bookable' post.
*/
function add_label_to_post_title( $title = '' ) {
	if ( is_bookable() ) return $title." <div class='redtext'>".__("bookable event", 'bookable_events')."</div>";
	else return $title;
}

/*
	Function to be run when the user clicks on the registration link. 
	It checks the user is logged and, if so, adds it to the event participants.
*/
function register_to_event( $content = '' ) {
	global $participant_key;
	global $bookable;
	global $postid_clicked;
	if ( $bookable && is_user_logged_in() ) {
		$fs_current_user = wp_get_current_user();
		$mykey_values = get_post_custom_values($participant_key);
		$meta_value = $fs_current_user->user_login;
		$unique = false;
			
		$user_in_array = in_array( $meta_value, (array)$mykey_values );
		
		if ( !$user_in_array && ( get_the_ID() == $postid_clicked ) ) {
			$content = "$content<br/><div class='mgs_t1'>$fs_current_user->user_login, ".__("thank you for the registration!", 'bookable_events')."</div><br/>";
			add_post_meta($postid_clicked, $participant_key, $meta_value, $unique);	

			// Before adding the first participant, the array is empty, so we have to recall the updated array
			$mykey_values = get_post_custom_values($participant_key); 
				
			$num_partecipanti = count($mykey_values);
			
			if ($num_partecipanti) {
				$content="$content<br/>".__("Registrered users for this event:", 'bookable_events');
				for ($i=0; $i<$num_partecipanti; $i++) {
					if ( $i == $num_partecipanti-1 ) $content = "$content ".$mykey_values[$i]." (".__("in total", 'bookable_events')." ".$num_partecipanti.").";
					else $content = "$content ".$mykey_values[$i].",";
				}
			} else {
				$content="$content<br/>".__("No registered users for this event.", 'bookable_events');
			}
			
			$content = "$content<br/><br/>";

		} elseif ( $user_in_array && ( get_the_ID() == $postid_clicked ) ) {
			$content = "$content<br/><div class='mgs_t1'>$fs_current_user->user_login, ".__("you are already registered!", 'bookable_events')."</div><br/>";
		}

	} elseif ( $bookable && !is_user_logged_in() ) {
		$content = "$content<br/><div class='mgs_t1'>".__("You must be logged in to register for this event!", 'bookable_events')."</div><br/>";
	}
	return $content;
}

/*
	Function to be run when the user clicks on the unregistration link.
	It checks the user is logged and, if so, removes it from the event participants.
*/
function unregister_from_event( $content = '' ) {
	global $participant_key;
	global $bookable;
	global $postid_clicked;

	if ( $bookable && is_user_logged_in() ) {
		$fs_current_user = wp_get_current_user();
		$mykey_values = get_post_custom_values($participant_key);
		$meta_value = $fs_current_user->user_login;
			
		$user_in_array = in_array( $meta_value, (array)$mykey_values );
		if ( !$user_in_array && ( get_the_ID() == $postid_clicked ) ) {
			$content = "$content<br/><div class='mgs_t1'>$fs_current_user->user_login, ".__("you have never been registered for this event!", 'bookable_events')."</div><br/>";
		} elseif ( $user_in_array && ( get_the_ID() == $postid_clicked ) ) {
			delete_post_meta($postid_clicked, $participant_key, $meta_value);
			$content = "$content<br/><div class='mgs_t1'>$fs_current_user->user_login, ".__("you have been unregistered!", 'bookable_events')."</div><br/>";

			// We have to recall the updated array for displaying the new participants' number.
			$mykey_values = get_post_custom_values($participant_key); 
				
			$num_partecipanti = count($mykey_values);
			
			if ($num_partecipanti) {
				$content="$content<br/>".__("Registrered users for this event:", 'bookable_events');
				for ($i=0; $i<$num_partecipanti; $i++) {
					if ( $i == $num_partecipanti-1 ) $content = "$content ".$mykey_values[$i]." (".__("in total", 'bookable_events')." ".$num_partecipanti.").";
					else $content = "$content ".$mykey_values[$i].",";
				}
			} else {
				$content="$content<br/>".__("No registered users for this event.", 'bookable_events');
			}
			
			$content = "$content<br/><br/>";
		}
	
	} elseif ( $bookable && !is_user_logged_in() ) {
		$content = "$content<br/><div class='mgs_t1'>".__("You must be logged in to unregister from this event!", 'bookable_events')."</div><br/>";
	}
	return $content;
}

function my_css() {
	if ( !defined('WP_PLUGIN_URL') ) $my_css_file = get_bloginfo( 'url' )."/wp-content/plugins/bookable_events/bookable_events.css";
	else $my_css_file = WP_PLUGIN_URL.'/bookable_events/bookable_events.css';
	echo "<link type='text/css' rel='stylesheet' href='$my_css_file' />";
}

/* 
	Registration of the plugin function 'bookable_events_activate' to be run when the plugin is activated. 
*/
register_activation_hook( __FILE__, 'bookable_events_activate' );

/* 
	Registration of the plugin function 'bookable_events_deactivate' to be run when the plugin is deactivated. 
*/
register_deactivation_hook( __FILE__, 'bookable_events_deactivate' );


/*
	Loads my css 
*/
add_action( 'wp_head', 'my_css' );

/* 
	Addition of the filter function 'add_label_to_post_title' to the 'the_title' (so the post title is modified as it is displayed in the browser screen).
	The 'bookable event' label is added to the title of a 'bookable' post.
*/
add_filter( 'the_title', 'add_label_to_post_title' );

/* 
	Addition of the filter function 'add_links_to_post' to the 'the_content' (so the post content is modified as it is displayed in the browser screen).
	The registration/unregistration links are added to a 'bookable' post.
*/
add_filter( 'the_content', 'add_links_to_post' );

/* 
	Addition of the filter function 'register_to_event' to the 'the_content' if the user clicks on the registration link.
*/
if ($click_register) {
	add_filter( 'the_content', 'register_to_event' );
}

/* 
	Addition of the filter function 'unregister_from_event' to the 'the_content' if the user clicks on the unregistration link.
*/
if ($click_unregister) {
	add_filter( 'the_content', 'unregister_from_event' );
}

?>